%% correlation_norm.m
%% Herve Lombaert, Jeremie Theriault, December 2th, 2005
%%
%% Correlation between two images
%% -a: image a
%% -b: image b
%% The two images must have the same size

function CC = correlation_norm(a, b)

    if size(a) ~= size(b), disp('a,b must have the same size'); end;

    a = im2double(a(:));
    b = im2double(b(:));
    overlap = (a>0).*(b>0);
    overlapa = a .* overlap; na = sum(a>0);
    overlapb = b .* overlap; nb = sum(b>0);

    % correlation
    ma = overlapa - sum(overlapa)/na;
    mb = overlapb - sum(overlapb)/nb;

    corr = (ma' * mb);
    
    % autocorrelation
    autocorrma = (ma' * ma);
    autocorrmb = (mb' * mb);
    
    % correlation coefficiant
    CC = corr / sqrt(autocorrma * autocorrmb);
    
    % correction, if one autocorrelation is zero, coefficiant is zero
    CC(isinf(CC)) = 0;

end
