%% simpowell.m
%% Jeremie Theriault
%%
%% Optimisation selon la methode de Powell
%% Calcul_Coef: fonction de la mesure de recalage
%% ima1: image 1
%% ima2: image 2
%% xytdeb: parametres initiaux pour la translation et la rotation

function [x,y,t] = simpowell(Calcul_Coef, ima1, ima2, xdeb, ydeb, tdeb)

% Initialisation des valeurs
bornexinf = floor(-size(ima1,2)/2);
bornexsup = floor(size(ima1,2)/2);

borneyinf = floor(-size(ima1,1)/2);
borneysup = floor(size(ima1,1)/2);

bornetinf = -180;
bornetsup = 180;

idx = 0; idy = 0; idt = 0;
x = xdeb; y = ydeb; t = tdeb;

test = transformation(ima2, x,y,t);
corr = Calcul_Coef(ima1, test);

NbIter = 0;

% Boucle de recherche du maximum
while ~(idx == 1 && idy == 1 && idt == 1)
    % Mise  jour
    NbIter = NbIter + 1;
    idx = 0;
    idy = 0;
    idt = 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Travail sur X
    % Deplacement negatif et positif
    stepinf = floor((x - bornexinf)/2);
    stepsup = floor((bornexsup - x)/2);

    % Correlation negative et positive
    test = transformation(ima2, x-stepinf,y,t);
    corrinf = Calcul_Coef(ima1, test);
    test = transformation(ima2, x+stepinf,y,t);
    corrsup = Calcul_Coef(ima1, test);

    % Mise a jour de la correlation max et des bornes
    [bornexinf, bornexsup, corr, corrinf, corrsup, x, stepinf, stepsup, idx] = MAJ_Bornes(bornexinf, bornexsup, corr, corrinf, corrsup, x, stepinf, stepsup, idx);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Test sur Y
    % Deplacement negatif et positif
    stepinf = floor((y - borneyinf)/2);
    stepsup = floor((borneysup - y)/2);

    % Correlation negative et positive
    test = transformation(ima2, x,y-stepinf,t);
    corrinf = Calcul_Coef(ima1, test);
    test = transformation(ima2, x,y+stepinf,t);
    corrsup = Calcul_Coef(ima1, test);

    % Mise a jour de la correlation max et des bornes
    [borneyinf, borneysup, corr, corrinf, corrsup, y, stepinf, stepsup, idy] = MAJ_Bornes(borneyinf, borneysup, corr, corrinf, corrsup, y, stepinf, stepsup, idy);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Test sur t
    % Deplacement negatif et positif
    stepinf = floor((t - bornetinf)/2);
    stepsup = floor((bornetsup - t)/2);

    % Correlation negative et positive
    test = transformation(ima2, x,y,t-stepinf);
    corrinf = Calcul_Coef(ima1, test);
    test = transformation(ima2, x,y,t+stepinf);
    corrsup = Calcul_Coef(ima1, test);
    
    % Mise a jour des bornes
    [bornetinf, bornetsup, corr, corrinf, corrsup, t, stepinf, stepsup, idt] = MAJ_Bornes(bornetinf, bornetsup, corr, corrinf, corrsup, t, stepinf, stepsup, idt);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Affichage
    fprintf(1,'Iteration %d\n', NbIter);
    fprintf(1,'x = %d y = %d t = %d f=%f\n', x, y, t, corr);
end
out = [x, y, t];

